function[zsol, Ssol] = HankelsquareLoss_dual_get_zS(U, C, y, options, L, zinit, Sinit)
    [d, r] = size(U);
    T = length(y) + 1 - d;
    
    if ~exist('zinit', 'var') || ~exist ('Sinit','var')
        zinit = zeros(d + T - 1, 1);
        Sinit = zeros(d, T);
    end
    
    
    %     % We solve the following system using pcg.
    %     myproj(z/2C, UUt S + reg S) = myproj(y, 0).
    
    rhs.z = y;
    rhs.S = zeros(size(Sinit));
    rhs = myproj(rhs, options);
    RHS = [rhs.z; rhs.S(:)]; % proj
    
    
    % Call PCG
    tol_omegax_pcg = options.innerpcgtol; 
    max_iterations_pcg = options.innerpcgmaxiter;
    
    %     % With matrix preconditioner
    %     if ~exist('L','var') || isempty(L)
    %         M = (eye(d + T -1 + d*T))/(2*C);
    %         M(d + T:end, d+T:end) = kron(eye(T), U*U' + 1e-5*eye(d));
    %         M = sparse(M);
    %         L = ichol(M);
    %     end
    %     [zSsol, unused, relres, iter] = pcg(@compute_matrix_system, RHS, tol_omegax_pcg, max_iterations_pcg,L,L',[zinit; Sinit(:)]);
    %     iter
    
    
    % With precon function handle
    if ~exist('L','var') || isempty(L)
        UtU = U'*U;
        L = (eye(r)  + (UtU)/options.preconreg);
    end
    [zSsol, unused, relres, iter] = pcg(@compute_matrix_system, RHS, tol_omegax_pcg, max_iterations_pcg, @myprecon,[],[zinit; Sinit(:)]);
     
     
    %     % Without preconditioner
    %     tic;
    %     [zSsol, unused, relres, iter] = pcg(@compute_matrix_system, RHS, tol_omegax_pcg, max_iterations_pcg,[],[],[zinit; Sinit(:)]);
    %     toc;
    %     iter
    
    function LHSx = compute_matrix_system(zSx)
        zx = zSx(1 : d + T - 1);
        Sx = reshape(zSx(d + T : end), d, T);
        
        %         x.z = zx;
        %         x.S = Sx;
        %         x1 = x;
        %         x = myproj(x, options);
        %         zx = x.z;
        %         Sx = x.S;
        %         sqrt(norm(x.z - x1.z)^2 + norm(x.S(:) - x1.S(:))^2)
        
        
        lhs.z = zx/(2*C);
        lhs.S = U*(U'*Sx) + options.reg*Sx;
        lhs = myproj(lhs, options);
        LHSx = [lhs.z; lhs.S(:)];
        
        %         % Sanity checks.
        %         disp(norm(LHSx - RHS)/norm(RHS));
    end
    
    
    function PzSx = myprecon(zSx)
        zx = zSx(1 : d + T - 1);
        Sx = reshape(zSx(d + T : end), d, T);
        
        %         % Sanity check.
        %         x.z = zx;
        %         x.S = Sx;
        %         x1 = myproj(x, options);
        %         sqrt(norm(x1.z - x.z)^2 + norm(x1.S - x.S, 'fro')^2)
        
        Pzx = (2*C)*zx;
        PSx =  (Sx - U*((options.preconreg*L)\(U'*Sx)))/(options.preconreg);  
        
        %         % Check
        %         UUt = U*U';
        %         PSx1 = (UUt +(options.preconreg)*eye(d))\Sx;
        %         norm(PSx - PSx1, 'fro')
        
        Px.z = Pzx;
        Px.S = PSx;
        Px = myproj(Px, options);
        %         norm(Px.z - Px1.z)
        PzSx = [Px.z; Px.S(:)];
    end
    
    zsol = zSsol(1 : d + T -1);
    Ssol = reshape(zSsol(d + T : end), d, T);
    
    xsol.z = zsol;
    xsol.S = Ssol;
    xsol = myproj(xsol, options);
    zsol = xsol.z;
    Ssol = xsol.S;
    
    %     % Sanity checks.
    %     ssol = Ssol(:);
    %     Hssol = zeros(size(zsol));
    %     for t = 1:(d+T-1)
    %         [I, J] = getindices(t+1, d, T);
    %         idx_t = sub2ind([d T], I, J);
    %         Hssol(t) = sum(ssol(idx_t));
    %     end
    %     norm(-Hssol + zsol, 'fro') % This should be zero.
    
end