function [Adot, UtAdot, UdottA] = squareLoss_dual_get_Zdot(U,lambda,I, J,mask,Udot,A,UtA,Achol)
    Udott = Udot.';
    Ut = U.';
    numThreads = 4;% parallelizing some components of the inner problem over tasks. Number should be increased if system supports it. 
    parallel_flag = true;% default value should be true. It is advised to contact authors if one is not able to run the code with parallel_flag as true. 
    % Computation of UdottA
    UdottA = mymultfullsparse(Udott, A, mask); % |Omega| operations. Mex code heavy.
    % Computation of -(UUdottA + UdotUtA) directly
    if parallel_flag
        UdotUtAplusUUdottA = -myspmaskmult_parallel([Ut;Udott], [UdottA; UtA], I, J, numThreads); % |Omega| operations. Mex code heavy.
    else
        UdotUtAplusUUdottA = -myspmaskmult([Ut;Udott], [UdottA; UtA], I, J); % |Omega| operations. Mex code heavy.
    end
    UomegatUdotUtAplusUUdottA = mymultfullsparse(Ut, UdotUtAplusUUdottA, mask); % |Omega| operations. Mex code heavy.
    if parallel_flag
        UtUinv_UdotUtAplusUUdottA = blocksolve_parallel(Achol, UomegatUdotUtAplusUUdottA, numThreads);
        UUtUinv_UdotUtAplusUUdottA = myspmaskmult_parallel(Ut, UtUinv_UdotUtAplusUUdottA, I, J, numThreads); % |Omega| operations. Mex code heavy.
    else
        UtUinv_UdotUtAplusUUdottA = blocksolve(Achol, UomegatUdotUtAplusUUdottA);
        UUtUinv_UdotUtAplusUUdottA = myspmaskmult(Ut, UtUinv_UdotUtAplusUUdottA, I, J); % |Omega| operations. Mex code heavy.
    end
    Adot = (2*lambda)*(UdotUtAplusUUdottA - UUtUinv_UdotUtAplusUUdottA);
    UtAdot = mymultfullsparse(Ut, Adot, mask); % |Omega| operations. Mex code heavy.
end

function X = blocksolve_parallel(R, B, numThreads)
    % R is a cell containing n upper triangular square matrices of size r.
    % B is a matrix of size r x n. X is a matrix of size r x n where the i-th
    % column is the solution of the equation A{i}x = B(:, i), with
    % A{i} = R{i}.'*R{i}. This is: the R{i} matrices are the upper triangular
    % Cholesky factors of the matrices A{i} (which we do not require/store).
    
    X = mycholsolvecell_parallel(R, B, numThreads);    
end

function X = blocksolve(R, B)
    % R is a cell containing n upper triangular square matrices of size r.
    % B is a matrix of size r x n. X is a matrix of size r x n where the i-th
    % column is the solution of the equation A{i}x = B(:, i), with
    % A{i} = R{i}.'*R{i}. This is: the R{i} matrices are the upper triangular
    % Cholesky factors of the matrices A{i} (which we do not require/store).
    
    X = mycholsolvecell(R,B);
end
