clear; clc; close all;

%% Load MovieLens 1M data that has users as rows and movies as columns.
dataset = 'data_ml_1m.mat';
load(dataset);

data_real_org = data_ml_1m;
data_real.rows = data_real_org.cols; % Transpose operation % Movies

data_real.cols = data_real_org.rows; % Users as columns.
data_real.entries = data_real_org.entries; % Ratings

% Movies as rows. Replace movie ids by integers.
[rows_sorted, ~, IC] = unique(data_real.rows);
movie_ids = 1:length(rows_sorted);
tmp = movie_ids(IC);
data_real.rows = tmp;

% %% Remove the ones with entries zero
% rm_ind = find(data_real.entries == 0);
% data_real.entries(rm_ind) =[];
% data_real.rows(rm_ind) =[];
% data_real.cols(rm_ind) =[];

clear tmp data_real_org data_ml_1m rows_sorted IC

%% Train / test split
hyper_randomize_seed = 2;
rng('default');
rng(hyper_randomize_seed); % seeding so that the splits are same across algorithms
randomize_seeds = randperm(100,10);
numSplits = 1;
prop_known = 0.8; % Training fraction

for sp = 1:numSplits
    fprintf('Dataset: %s, Split: %d of %d\n', dataset, sp, numSplits);
    fprintf('randomize_seed: %d, prop_known: %f\n', randomize_seeds(sp), prop_known);
    rng('default');
    rng(randomize_seeds(sp)); % seeding so that the splits are same across algorithms

    % permute them
    random_order = randperm(length(data_real.rows));
    data_real.rows = data_real.rows(random_order);
    data_real.cols = data_real.cols(random_order);
    data_real.entries = data_real.entries(random_order);

    m = max(data_real.rows);
    n = max(data_real.cols);

    % Data training
    allidx = 1 : length(data_real.rows); % all indices

    M = round(prop_known*length(data_real.rows)); % Number of entries known
    idx = 1 : M; % indices for training
    Matcom.row = (data_real.rows(idx))';
    Matcom.col = (data_real.cols(idx))';
    Matcom.values_Omega = (data_real.entries(idx))';

    S = sparse(Matcom.row', Matcom.col', Matcom.values_Omega', m, n);
    [I_0, J_0, data_0] = find(S);
    Matcom.row = I_0';
    Matcom.col = J_0';
    Matcom.values_Omega = data_0';
    M = length(Matcom.row);

    data_ls.rows = Matcom.row';
    data_ls.cols = Matcom.col';
    data_ls.entries = Matcom.values_Omega';
    data_ls.nentries = length(data_ls.rows);

    % Test data
    M_test = length(data_real.rows) - M;
    L = length(data_real.rows);
    idx_test = allidx(L - M_test + 1: L);

    data_ts.rows = (data_real.rows(idx_test));
    data_ts.cols = (data_real.cols(idx_test));
    data_ts.entries = (data_real.entries(idx_test));

    C = 1e4;
    fprintf('C=%e\n',C);

    % Rank that we impose
    rank = 5;
    fprintf('rank=%d\n',rank);

    % Problem structure
    problem.d = m;
    problem.T = n;
    problem.r = rank;
    problem.C = C;
    problem.I = uint32(data_ls.rows);
    problem.J = uint32(data_ls.cols);
    problem.Y = data_ls.entries;
    problem.I_test = uint32(data_ts.rows');
    problem.J_test = uint32(data_ts.cols);
    problem.Y_test = data_ts.entries;


    % All options are not mandatory
    opt.maxiter = 300;
    opt.tolgradnorm = 1e-6;
    % opt.maxinner = 30;
    opt.tolrelgradnorm = 1e-10;
    opt.method = 'cg';
    opt.randome_initialization = true;
    opt.maxinnerepoch = 10;
    opt.innertolg = 1e-11;
    opt.finalmaxinnerepoch = 1000;
    opt.finalinnertolg = 1e-10;
    opt.sparse_s_flag = 2;
    opt.finalsparse_s_flag = 0;
    opt.num_omega_s = 60000;
    opt.finalnum_omega_s = 60000;
    opt.flag_parallel = true;
    opt.numThreads = 25;
    % opt.Delta0 = 100/8/4/4/4;
    
    fprintf('Dimensions d and T: %d, %d, numOmegaS: %d, finalnumOmegaS: %d\n',...
        m,n,opt.num_omega_s,opt.finalnum_omega_s);

    opt % printing opt
    % Initialization
    initstart = tic;
    if ~opt.randome_initialization
        mysparsemat = sparse(double(problem.I), double(problem.J),ones(length(problem.I), 1), problem.d, problem.T, length(problem.I));
        mysetsparseentries(mysparsemat , problem.Y);
        [Uinit, ~, ~] = svds(mysparsemat, problem.r);
        Uinit = Uinit/norm(Uinit, 'fro');
        fprintf('SVD based initialization with rank = %d\n', problem.r);
    else
        Uinit = [];
    end
    inittime = toc(initstart);
    
    % Run proposed algorithm
    [Lopt, Ropt_temp, infos, options, ~, ~,mask] = fixedrank_matrixcompletion_nnmc_new_sparse_s_memory(problem, [],opt);
    
    LR = Lopt*Ropt_temp;
    fprintf('Before refining: Minimum value: %e, Maximum value: %e\n',min(LR(:)),max(LR(:)));
    
%     mask = sparse(double(problem.I), double(problem.J), ones(length(problem.I),1), problem.d, problem.T, length(problem.I));
    refining = tic;
    [zopt, sopt, UtZ_UtAtSopt, masksopt] = ...
        solve_nnmc_sparse_s(Lopt, problem.C, problem.I, problem.J, problem.Y, mask, ...
        options.finalmaxinnerepoch, options.innertolg, options.finalsparse_s_flag, ...
        options.finalnum_omega_s, options.flag_parallel, options.numThreads);
    refining_time = toc(refining);
    fprintf('refining_time: %e\n',refining_time);
    Ropt = UtZ_UtAtSopt;
    LR = Lopt*Ropt;
    
    % Compute final RMSE
    error_train = myspmaskmult(Lopt', Ropt, problem.I, problem.J) - problem.Y;
    square_loss_train = norm(error_train)^2; % Square error on train data.
    final_loss_train_rmse = sqrt((error_train'*error_train)/length(problem.Y)); % RMSE on train data.

%     if exist('I_test', 'var')
    error_test = myspmaskmult(Lopt', Ropt, problem.I_test, problem.J_test) - problem.Y_test;
    loss_test = norm(error_test)^2; % Square error on test data.
    final_loss_test_rmse = sqrt((error_test'*error_test)/length(problem.Y_test)); % RMSE on test data.

    final_primal_objective = 0.5*norm(UtZ_UtAtSopt,'fro')^2 + C*square_loss_train;

    fprintf('After refining: Minimum value: %e, Maximum value: %e\n',min(LR(:)),max(LR(:)));
    fprintf('After refining: PrimalObj: %e LossRMSE %e TestLossRMSE %e \n', final_primal_objective, final_loss_train_rmse, final_loss_test_rmse);
%     end
    
    proposed_time = [infos.time];
    proposed_time = proposed_time + inittime;
    time_after_refining = proposed_time(end) + refining_time;
    proposed_time = [proposed_time,time_after_refining];

    proposed_test_rmse = [infos.loss_test_rmse];
    proposed_test_rmse = [proposed_test_rmse,final_loss_test_rmse];

    proposed_train_rmse = [infos.loss_rmse];
    proposed_train_rmse = [proposed_train_rmse, final_loss_train_rmse];
    
    primal_objective = [infos.primal_objective];
    primal_objective = [primal_objective, final_primal_objective];

end
